;Author and Copyright 2009: meingbg, email: meingbg at gmail dot com
;License: BSD

;cellphonemode.el is not a real mode for emacs; rather, it's a
;startup script for making it easier to work in emacs on a cell
;phone (through ssh etc)

;When you're on a cell phone, type M-x cellphone-on <RET>

;functions to make the letter keys have different meaning dependent on
;which cellphone keymap is active
(defun cellphone-switch-map (mapname)
  (let ((map (member mapname cellphone-maps)))
    (when map
      (cellphone-cleanmap)
      (cellphone-bind (cadr map))
      (setq cellphone-current-map mapname)
      (message "%s" (concat "KEYBOARD: " mapname)))))
(defun cellphone-bind (bindings)
  (when (cdr bindings)
    (let ((key (car bindings)) (bind (cadr bindings)))
      (cond ((symbolp bind) (global-set-key key bind))
            ((or (vectorp bind) (stringp bind))
             (let ((bf (key-binding bind)))
               (if (symbolp bf) (global-set-key key bf))))))
    (cellphone-bind (cddr bindings))))
(defun cellphone-cleanmap ()
  (mapcar (lambda (x) (global-set-key x 'self-insert-command))
          '("a" "b" "c" "d" "e" "f" "g" "h" "i" "j" "k" "l" "m"
            "n" "o" "p" "q" "r" "s" "t" "u" "v" "w" "x" "y" "z"
            "0" "1" "2" "3" "4" "5" "6" "7" "8" "9" "," ".")))
(defun cellphone-switch-map-next ()
  (interactive)
  (let ((a (apply #'list cellphone-maps)))
    (setcdr (last a) a)
    (cellphone-switch-map (nth 2 (member cellphone-current-map a)))))
(defun cellphone-switch-map-previous ()
  (interactive)
  (let ((a (apply #'list (reverse cellphone-maps))))
    (setcdr (last a) a)
    (cellphone-switch-map (nth 2 (member cellphone-current-map a)))))
(setq cellphone-off-menu-bar-mode menu-bar-mode)

;Make left and right arrow keys switch cellphone keymap
(defun cellphone-on ()
  (interactive)
  (menu-bar-mode 0)
  (global-set-key [right] 'cellphone-switch-map-next)
  (global-set-key [left] 'cellphone-switch-map-previous)
  (cellphone-switch-map (car cellphone-maps)))
(defun cellphone-off ()
  (interactive)
  (cellphone-cleanmap)
  (menu-bar-mode (if cellphone-off-menu-bar-mode 1 0))
  (global-set-key [right] 'forward-char)
  (global-set-key [left] 'backward-char))



;Define cellphone keymaps                  <<< This is where you CUSTOMIZE
(setq cellphone-current-map "off"
;Each cellphone keymap is defined by two items in cellphone-maps.
;First the name as a string, then a list defining the bindings.
;Each binding is defined by two items in the binding list. First
;a string containing the character two be bound to a function,
;then either a function name it should be bound to or a key
;combination that currently is bound to the desired function.
      cellphone-maps '(
"off"
()
"Movement"
("i" [?\C-p] "j" [?\C-b] "k" [?\C-n] "l" [?\C-f])
"orgmode"
("f" [?\C-c ?\C-n] "d" [?\C-c ?\C-p] "r" [?\C-c ?\C-f] "e" [?\C-c ?\C-b] "u" [?\C-c ?\C-u] "n" [C-return] "h" [?\C-c return] "i" [M-S-up] "j" [M-S-left] "k" [M-S-down] "l" [M-S-right] "w" [?\C-c ?\C-x ?\C-w] "y" [?\C-c ?\C-x ?\C-y] "s" [?\C-c ?/ ?r] "a" [?\C-c ?/] "o" [?\C-c ?\C-o] "t" [?\C-c ?\C-t] "8" [?\C-c ?\C-x ?\C-i] "9" [?\C-c ?\C-x ?\C-o] "7" [?\C-c ?\C-x ?\C-r] "0" fixa-org-klocka "g" [tab] "b" [S-tab])

))
